'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.find = find;

var _graphqlLanguageServiceUtils = require('graphql-language-service-utils');

var _babylon = require('babylon');

// import crypto from 'crypto';
// import {parse} from 'graphql';
// import path from 'path';
// import util from 'util';

// Attempt to be as inclusive as possible of source text.
/**
 *  Copyright (c) Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the license found in the
 *  LICENSE file in the root directory of this source tree.
 *
 *  
 */

var BABYLON_OPTIONS = {
  allowImportExportEverywhere: true,
  allowReturnOutsideFunction: true,
  allowSuperOutsideMethod: true,
  sourceType: 'module',
  plugins: [
  // Previously "*"
  'asyncGenerators', 'classProperties', 'decorators', 'doExpressions', 'dynamicImport', 'exportExtensions', 'flow', 'functionBind', 'functionSent', 'jsx', 'objectRestSpread'],
  strictMode: false
};

function find(text, filePath) {
  var result = [];
  var ast = (0, _babylon.parse)(text, BABYLON_OPTIONS);
  // const moduleName = extractModuleName(text, filePath);

  var visitors = {
    CallExpression: function CallExpression(node) {
      var callee = node.callee;
      if (!(callee.type === 'Identifier' && CREATE_CONTAINER_FUNCTIONS[callee.name] || callee.kind === 'MemberExpression' && callee.object.type === 'Identifier' && callee.object.value === 'Relay' && callee.property.type === 'Identifier' && CREATE_CONTAINER_FUNCTIONS[callee.property.name])) {
        traverse(node, visitors);
        return;
      }
      var fragments = node.arguments[1];
      if (fragments.type === 'ObjectExpression') {
        fragments.properties.forEach(function (property) {
          // invariant(
          //   property.type === 'ObjectProperty' &&
          //     property.key.type === 'Identifier' &&
          //     property.value.type === 'TaggedTemplateExpression',
          //   'FindGraphQLTags: `%s` expects fragment definitions to be ' +
          //     '`key: graphql`.',
          //   node.callee.name,
          // );
          // const keyName = property.key.name;
          var tagName = getGraphQLTagName(property.value.tag);
          // invariant(
          //   tagName,
          //   'FindGraphQLTags: `%s` expects fragment definitions to be tagged ' +
          //     'with `graphql`, got `%s`.',
          //   node.callee.name,
          //   getSourceTextForLocation(text, property.value.tag.loc),
          // );
          var template = getGraphQLText(property.value.quasi);
          if (tagName === 'graphql' || tagName === 'graphql.experimental') {
            // validateTemplate(template, moduleName, keyName);
            var loc = property.value.loc;
            var _range = new _graphqlLanguageServiceUtils.Range(new _graphqlLanguageServiceUtils.Position(loc.start.line - 1, loc.start.column), new _graphqlLanguageServiceUtils.Position(loc.end.line - 1, loc.end.column));
            result.push({
              tag: tagName,
              template: template,
              range: _range
            });
          }
        });
      } else {
        // invariant(
        //   fragments && fragments.type === 'TaggedTemplateExpression',
        //   'FindGraphQLTags: `%s` expects a second argument of fragment ' +
        //     'definitions.',
        //   node.callee.name,
        // );
        var tagName = getGraphQLTagName(fragments.tag);
        // invariant(
        //   tagName,
        //   'FindGraphQLTags: `%s` expects fragment definitions to be tagged ' +
        //     'with `graphql`, got `%s`.',
        //   node.callee.name,
        //   getSourceTextForLocation(text, fragments.tag.loc),
        // );
        var _template = getGraphQLText(fragments.quasi);
        if (tagName === 'graphql' || tagName === 'graphql.experimental') {
          // validateTemplate(template, moduleName);
          var loc = fragments.loc;
          var _range2 = new _graphqlLanguageServiceUtils.Range(new _graphqlLanguageServiceUtils.Position(loc.start.line - 1, loc.start.column), new _graphqlLanguageServiceUtils.Position(loc.end.line - 1, loc.end.column));
          result.push({
            tag: tagName,
            template: _template,
            range: _range2
          });
        }
      }

      // Visit remaining arguments
      for (var ii = 2; ii < node.arguments.length; ii++) {
        visit(node.arguments[ii], visitors);
      }
    },
    TaggedTemplateExpression: function TaggedTemplateExpression(node) {
      var tagName = getGraphQLTagName(node.tag);
      if (tagName != null) {
        // const template = getGraphQLText(node.quasi);
        if (tagName === 'graphql' || tagName === 'graphql.experimental') {
          // validateTemplate(template, moduleName);
          var loc = node.loc;
          var _range3 = new _graphqlLanguageServiceUtils.Range(new _graphqlLanguageServiceUtils.Position(loc.start.line - 1, loc.start.column), new _graphqlLanguageServiceUtils.Position(loc.end.line - 1, loc.end.column));
          result.push({
            tag: tagName,
            template: node.quasi.quasis[0].value.raw,
            range: _range3
          });
        }
      }
    }
  };
  visit(ast, visitors);
  return result;
}

// const cache = {};
// export function memoizedFind(
//   text: string,
//   filePath: string,
// ): Array<{tag: string, template: string}> {
//   const hash = crypto
//     .createHash('md5')
//     .update(filePath)
//     .update(text)
//     .digest('hex');
//   let result = cache[hash];
//   if (!result) {
//     result = find(text, filePath);
//     cache[hash] = result;
//   }
//   return result;
// }

var CREATE_CONTAINER_FUNCTIONS = {
  createFragmentContainer: true,
  createPaginationContainer: true,
  createRefetchContainer: true
};

var IDENTIFIERS = { graphql: true };

var IGNORED_KEYS = {
  comments: true,
  end: true,
  leadingComments: true,
  loc: true,
  name: true,
  start: true,
  trailingComments: true,
  type: true
};

function getGraphQLTagName(tag) {
  if (tag.type === 'Identifier' && IDENTIFIERS.hasOwnProperty(tag.name)) {
    return tag.name;
  } else if (tag.type === 'MemberExpression' && tag.object.type === 'Identifier' && tag.object.name === 'graphql' && tag.property.type === 'Identifier' && tag.property.name === 'experimental') {
    return 'graphql.experimental';
  }
  return null;
}

function getGraphQLText(quasi) {
  var quasis = quasi.quasis;
  // invariant(
  //   quasis && quasis.length === 1,
  //   'FindGraphQLTags: Substitutions are not allowed in graphql tags.',
  // );
  return quasis[0].value.raw;
}

// function getSourceTextForLocation(text, loc) {
//   if (loc == null) {
//     return '(source unavailable)';
//   }
//   const lines = text.split('\n').slice(loc.start.line - 1, loc.end.line);
//   lines[0] = lines[0].slice(loc.start.column);
//   lines[lines.length - 1] = lines[lines.length - 1].slice(0, loc.end.column);
//   return lines.join('\n');
// }

// function validateTemplate(template, moduleName, keyName) {
//   const ast = parse(template);
//   ast.definitions.forEach((def: any) => {
//     invariant(
//       def.name,
//       'FindGraphQLTags: In module `%s`, a definition of kind `%s` requires a name.',
//       moduleName,
//       def.kind,
//     );
//     const definitionName = def.name.value;
//     if (def.kind === 'OperationDefinition') {
//       const operationNameParts = definitionName.match(
//         /^(.*)(Mutation|Query|Subscription)$/,
//       );
//       invariant(
//         operationNameParts && definitionName.startsWith(moduleName),
//         'FindGraphQLTags: Operation names in graphql tags must be prefixed ' +
//           'with the module name and end in "Mutation", "Query", or ' +
//           '"Subscription". Got `%s` in module `%s`.',
//         definitionName,
//         moduleName,
//       );
//     } else if (def.kind === 'FragmentDefinition') {
//       if (keyName) {
//         invariant(
//           definitionName === moduleName + '_' + keyName,
//           'FindGraphQLTags: Container fragment names must be ' +
//             '`<ModuleName>_<propName>`. Got `%s`, expected `%s`.',
//           definitionName,
//           moduleName + '_' + keyName,
//         );
//       } else {
//         invariant(
//           definitionName.startsWith(moduleName),
//           'FindGraphQLTags: Fragment names in graphql tags must be prefixed ' +
//             'with the module name. Got `%s` in module `%s`.',
//           definitionName,
//           moduleName,
//         );
//       }
//     }
//   });
// }

// function extractModuleName(text, filePath) {
//   const rawModuleName =
//     extractProvidesModuleName(text) || extractFileModuleName(filePath);
//   return rawModuleName.replace(/\.react$/, '').replace(/[^a-zA-Z0-9_]/g, '_');
// }

// function extractFileModuleName(filePath) {
//   const filename = path.basename(filePath, path.extname(filePath));
//   if (filename !== 'index') {
//     return filename;
//   }
//   return path.basename(path.dirname(filePath));
// }

// function extractProvidesModuleName(text) {
//   const propertyRegex = /@(\S+) *(\S*)/g;
//   let captures;
//   while ((captures = propertyRegex.exec(text))) {
//     const prop = captures[1];
//     const value = captures[2];
//     if (prop === 'providesModule') {
//       return value;
//     }
//   }
// }

// function invariant(condition, msg, ...args) {
//   if (!condition) {
//     throw new Error(util.format(msg, ...args));
//   }
// }

function visit(node, visitors) {
  var fn = visitors[node.type];
  if (fn != null) {
    fn(node);
    return;
  }
  traverse(node, visitors);
}

function traverse(node, visitors) {
  for (var key in node) {
    if (IGNORED_KEYS[key]) {
      continue;
    }
    var prop = node[key];
    if (prop && typeof prop === 'object' && typeof prop.type === 'string') {
      visit(prop, visitors);
    } else if (Array.isArray(prop)) {
      prop.forEach(function (item) {
        if (item && typeof item === 'object' && typeof item.type === 'string') {
          visit(item, visitors);
        }
      });
    }
  }
}